/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package libraryfitnesse;

import java.util.LinkedList;
import java.util.List;
import loans.AbstractLoan;
import loans.LongLoan;
import loans.ShortLoan;
import org.joda.time.DateTime;

/**
 *
 * @author student
 */
public class LoanManager {

    private List<AbstractLoan> loans;

    public LoanManager() {
        this.loans = new LinkedList<>();
    }

    public AbstractLoan createLongLoan(Reader reader, Book book, DateTime from) {
        if (isBookAvailableFrom(book, from)) {
            LongLoan loan = new LongLoan(book, reader, from);
            this.loans.add(loan);
            return loan;
        } else {
            throw new IllegalArgumentException("The selected book is not available in the given time");
        }

    }

    public AbstractLoan createShortLoan(Reader reader, Book book, DateTime from) {
        if (isBookAvailableFrom(book, from)) {
            ShortLoan loan = new ShortLoan(book, reader, from);
            this.loans.add(loan);
            return loan;
        } else {
            throw new IllegalArgumentException("The selected book is not available in the given time");
        }

    }

    private boolean isBookAvailableFrom(Book book, DateTime from) {
        AbstractLoan openLoan = findOpenLoanAt(book, from);
        if (openLoan == null) {
            return true;
        } else {
            return false;
        }
    }

    private List<AbstractLoan> findLoans(Book book) {
        List<AbstractLoan> foundLoans = new LinkedList();
        for (AbstractLoan loan : this.loans) {
            if (loan.belongsToBook(book)) {
                foundLoans.add(loan);
            }
        }
        return foundLoans;
    }

    private List<AbstractLoan> findLoans(Reader reader) {
        List<AbstractLoan> foundLoans = new LinkedList();
        for (AbstractLoan loan : this.loans) {
            if (loan.belongsToReader(reader)) {
                foundLoans.add(loan);
            }
        }
        return foundLoans;
    }

    public AbstractLoan findOpenLoanAt(Book book, DateTime time) {
        List<AbstractLoan> loans = findLoans(book);
        for (AbstractLoan loan : loans) {
            if (loan.isOpenAt(time)) {
                return loan;
            }
        }
        return null;
    }

    public List<AbstractLoan> findOpenLoansAt(Reader reader, DateTime time) {
        List<AbstractLoan> loans = findLoans(reader);
        List<AbstractLoan> found = new LinkedList<>();
        for (AbstractLoan loan : loans) {
            if (loan.isOpenAt(time)) {
                found.add(loan);
            }
        }
        return found;
    }

    public List<AbstractLoan> getOpenLoans(DateTime time) {
        List<AbstractLoan> found = new LinkedList<>();
        for (AbstractLoan loan : this.loans) {
            if (loan.isOpenAt(time)) {
                found.add(loan);
            }
        }
        return found;
    }

    public int getTotalOpenFines(Reader reader, DateTime time) {
        int total = 0;
        List<AbstractLoan> loans = findOpenLoansAt(reader, time);
        for (AbstractLoan loan : loans) {
            total += loan.getFees();
        }
        return total;
    }

    public int getTotalFines(Reader reader, DateTime time) {
        List<AbstractLoan> loans = findLoans(reader);
        int total = 0;
        for (AbstractLoan loan : loans) {
            if (loan.isClosed()) {
                total += loan.getFees();
            } else {
                total += loan.getPreliminaryFees(time);
            }
        }
        return total;

    }

    public AbstractLoan closeLoan(Book book, DateTime to) {
        AbstractLoan oldestLoan = findOpenLoanAt(book, to);

        oldestLoan.close(to);
        return oldestLoan;
    }

    public List<AbstractLoan> getAllLoans() {
        return this.loans;
    }

    public void reset() {
        this.loans.clear();
    }

}
